<?php
/**
*
* @package Moods
* @copyright (c) 2016 Theriddler - (http://www.peppersrus-forum.nl)
* @copyright (c) 2015 Rmcgirr83
* @Code lent from nationalflags extension by Rmcgirr83
* @license http://opensource.org/licenses/gpl-2.0.php GNU General Public License v2
*
*/

namespace theriddler\moods\controller;

/**
* Admin controller
*/
class admin_controller
{
	/** @var \phpbb\cache\service */
	protected $cache;

	/** @var \phpbb\config\config */
	protected $config;

	/** @var \phpbb\db\driver\driver_interface */
	protected $db;

	/** @var \phpbb\pagination */
	protected $pagination;

	/** @var \phpbb\controller\helper */
	protected $helper;

	/** @var \phpbb\request\request */
	protected $request;

	/** @var \phpbb\template\template */
	protected $template;

	/** @var \phpbb\user */
	protected $user;

	/** @var \phpbb\log\log */
	protected $log;

	/** @var \phpbb\extension\manager "Extension Manager" */
	protected $ext_manager;

	/** @var \phpbb\path_helper */
	protected $path_helper;

	/** @var string phpBB root path */
	protected $root_path;

	/** @var string phpEx */
	protected $php_ext;

	/**
	 * The database table the moods are stored in
	 *
	 * @var string
	 */
	protected $moods_table;

	/* @var \theriddler\moods\core\moods */
	protected $functions;

	/** @var string Custom form action */
	protected $u_action;

	/**
	* Constructor
	*
	* @param \phpbb\cache\service					$cache				Cache object
	* @param \phpbb\config\config					$config				Config object
	* @param \phpbb\db\driver\driver_interface		$db					Database object
	* @param \phpbb\pagination						$pagination			Pagination object
	* @param \phpbb\controller\helper           	$helper     	    Controller helper object
	* @param \phpbb\request\request					$request			Request object
	* @param \phpbb\template\template				$template			Template object
	* @param \phpbb\user							$user				User object
	* @param \phpbb\log								$log				Log object
	* @param \phpbb\extension\manager				$ext_manager		Extension manager object
	* @param \phpbb\path_helper						$path_helper		Path helper object
	* @param string                             	$root_path      	phpBB root path
	* @param string                             	$php_ext        	phpEx
	* @param string									$moods_table		Name of the table used to store mood data
	* @param \theriddler\moods\core\moods			$functions			Functions for the extension
	* @return \theriddler\moods\controller\admin_controller
	* @access public
	*/
	public function __construct(
			\phpbb\cache\service $cache,
			\phpbb\config\config $config,
			\phpbb\db\driver\driver_interface $db,
			\phpbb\pagination $pagination,
			\phpbb\controller\helper $helper,
			\phpbb\request\request $request,
			\phpbb\template\template $template,
			\phpbb\user $user,
			\phpbb\log\log $log,
			\phpbb\extension\manager $ext_manager,
			\phpbb\path_helper $path_helper,
			$root_path,
			$php_ext,
			$moods_table,
			\theriddler\moods\core\moods $functions)
	{
		$this->cache = $cache;
		$this->config = $config;
		$this->db = $db;
		$this->pagination = $pagination;
		$this->helper = $helper;
		$this->request = $request;
		$this->template = $template;
		$this->user = $user;
		$this->log = $log;
		$this->ext_manager	 = $ext_manager;
		$this->path_helper	 = $path_helper;
		$this->root_path = $root_path;
		$this->php_ext = $php_ext;
		$this->moods_table = $moods_table;
		$this->functions = $functions;

		$this->ext_path = $this->ext_manager->get_extension_path('theriddler/moods', true);
		$this->ext_path_web = $this->path_helper->update_web_root_path($this->ext_path);
	}

	/**
	 * Display the options a user can configure for this extension
	 *
	 * @return null
	 * @access public
	 */
	public function display_options()
	{
		// Create a form key for preventing CSRF attacks
		add_form_key('moods_settings');

		// Is the form being submitted to us?
		if ($this->request->is_set_post('submit'))
		{
			// Test if the submitted form is valid
			if (!check_form_key('moods_settings'))
			{
				trigger_error($this->user->lang['FORM_INVALID'] . adm_back_link($this->u_action));
			}

			if (!function_exists('validate_data'))
			{
				include($this->root_path . 'includes/functions_user.' . $this->php_ext);
			}

			$error = $this->user->lang[$error];

			if (!sizeof($error))
			{
				// Set the options the user configured
				$this->set_options();

				// Add option settings change action to the admin log
				$this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'MOOD_CONFIG_SAVED');

				// Option settings have been updated and logged
				// Confirm this to the user and provide link back to previous page
				trigger_error($this->user->lang('MOOD_SETTINGS_CHANGED') . adm_back_link($this->u_action));
			}
		}

		// Set output vars for display in the template
		$this->template->assign_vars(array(
			'ERROR'				=> isset($error) ? ((sizeof($error)) ? implode('<br />', $error) : '') : '',
			'MOODS_VERSION' 	=> $this->config['moods_version'],
			'ALLOW_MOODS'		=> $this->config['allow_moods'] ? true : false,
			'MOODS_DISPLAY_TO_GUESTS'	=> $this->config['moods_display_to_guests'] ? true : false,

			'S_MOODS'			=> true,

			'U_ACTION'			=> $this->u_action,
		));
	}

	/**
	 * Set the options a user can configure
	 *
	 * @return null
	 * @access protected
	 */
	protected function set_options()
	{
		$this->config->set('allow_moods', $this->request->variable('allow_moods', 0));
		$this->config->set('moods_display_to_guests', $this->request->variable('moods_display_to_guests', 0));
	}

	/**
	 * Display the moods
	 *
	 * @return null
	 * @access public
	 */
	public function display_moods()
	{
		$start = $this->request->variable('start', 0);
		$pagination_url = $this->u_action;

		$this->user->add_lang_ext('theriddler/moods', 'common');

		$sql = 'SELECT f.*, COUNT(u.user_mood) as user_count
			FROM ' . $this->moods_table . ' f
				LEFT JOIN ' . USERS_TABLE . " u on f.mood_id = u.user_mood
			GROUP BY f.mood_id
			ORDER BY f.mood_name ASC";
		$result = $this->db->sql_query_limit($sql, $this->config['topics_per_page'], $start);

		// for counting of all the moods
		// used for pagination
		$result2 = $this->db->sql_query($sql);
		$row2 = $this->db->sql_fetchrowset($result2);
		$total_count = (int) sizeof($row2);
		$this->db->sql_freeresult($result2);
		unset($row2);

		while ($row = $this->db->sql_fetchrow($result))
		{
			$user_count = $this->user->lang('MOOD_USERS', (int) $row['user_count']);

			$this->template->assign_block_vars('moods', array(
				'MOOD_NAME'		=> $row['mood_name'],
				'MOOD_IMG'		=> $this->ext_path_web . 'moods/' . strtolower($row['mood_image']),
				'MOOD_ID'		=> $row['mood_id'],
				'USER_COUNT'	=> $user_count,
				'U_EDIT'		=> $this->u_action . "&amp;mood_id={$row['mood_id']}&amp;action=edit",
				'U_DELETE'		=> $this->u_action . "&amp;mood_id={$row['mood_id']}&amp;action=delete",)
			);
		}
		$this->db->sql_freeresult($result);

		$start = $this->pagination->validate_start($start, $this->config['topics_per_page'], $total_count);
		$this->pagination->generate_template_pagination($pagination_url, 'pagination', 'start', $total_count, $this->config['topics_per_page'], $start);

		$this->template->assign_vars(array(
			'TOTAL_MOODS'	=> $total_count,
			'S_MOODS'	=> true,
		));
	}

	/**
	 * Add a mood
	 *
	 * @return null
	 * @access public
	 */
	public function add_mood()
	{
		// Add form key
		add_form_key('add_mood');

		$errors = array();

		$mood_row = array(
			'mood_name'			=> ucfirst(utf8_normalize_nfc($this->request->variable('mood_name', '', true))),
			'mood_image'		=> $this->request->variable('mood_image', ''),
		);

		if ($this->request->is_set_post('submit'))
		{
			$errors = $this->check_mood($mood_row['mood_image'], $mood_row['mood_name'], $errors, 'add_mood');

			if (!sizeof($errors))
			{
				$sql = 'INSERT INTO ' . $this->moods_table . ' ' . $this->db->sql_build_array('INSERT', $mood_row);
				$this->db->sql_query($sql);

				$this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'LOG_MOOD_ADD', time(), array($mood_row['mood_name']));

				$this->cache->destroy('_user_moods');
				// cache this data for ever, can only change in ACP
				$this->functions->cache_moods();

				trigger_error($this->user->lang['MSG_MOOD_ADDED'] . adm_back_link($this->u_action));
			}
		}

		$this->template->assign_vars(array(
			'U_ACTION'		=> $this->u_action . '&amp;action=add',
			'U_BACK'		=> $this->u_action,
			'MOOD_NAME'		=> $mood_row['mood_name'],
			'MOOD_IMAGE'	=> $mood_row['mood_image'],
			'ERROR_MSG'		=> (sizeof($errors)) ? implode('<br />', $errors) : '',

			'S_ADD_MOOD'	=> true,
			'S_ERROR'		=> (sizeof($errors)) ? true : false,
		));
	}

	/**
	 * Edit a mood
	 *
	 * @param int $mood_id The mood identifier to edit
	 * @return null
	 * @access public
	 */
	public function edit_mood($mood_id)
	{
		// Add form key
		add_form_key('edit_mood');

		$errors = array();

		$mood_row = array(
			'mood_name'			=> utf8_normalize_nfc($this->request->variable('mood_name', '', true)),
			'mood_image'		=> $this->request->variable('mood_image', ''),
		);

		if ($this->request->is_set_post('submit'))
		{
			$errors = $this->check_mood($mood_row['mood_image'], $mood_row['mood_name'], $errors, 'edit_mood');

			if (!sizeof($errors))
			{
				$sql = 'UPDATE ' . $this->moods_table . '
					SET ' . $this->db->sql_build_array('UPDATE', $mood_row) . '
					WHERE mood_id = ' . (int) $mood_id;
				$this->db->sql_query($sql);

				$this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'LOG_MOOD_EDIT', time(), array($mood_row['mood_name']));

				$this->cache->destroy('_user_moods');
				$this->functions->cache_moods();

				trigger_error($this->user->lang['MSG_MOOD_EDITED'] . adm_back_link($this->u_action));
			}
		}

		$sql = 'SELECT mood_id, mood_name, mood_image
			FROM ' . $this->moods_table . '
			WHERE mood_id =' . (int) $mood_id;
		$result = $this->db->sql_query($sql);
		$mood_row = $this->db->sql_fetchrow($result);
		$this->db->sql_freeresult($result);

		if (!$mood_row)
		{
			trigger_error($this->user->lang['MOOD_ERROR_NOT_EXIST'] . adm_back_link($this->u_action . '&amp;mode=manage'), E_USER_WARNING);
		}

		$this->template->assign_vars(array(
			'L_TITLE'		=> $this->user->lang['MOOD_EDIT'],
			'U_ACTION'		=> $this->u_action . "&amp;mood_id=$mood_id&amp;action=edit",
			'U_BACK'		=> $this->u_action . '&amp;mode=manage',
			'ERROR_MSG'		=> (sizeof($errors)) ? implode('<br />', $errors) : '',

			'MOOD_NAME'		=> $mood_row['mood_name'],
			'MOOD_IMAGE'	=> $mood_row['mood_image'],
			'MOOD_ID'		=> $mood_row['mood_id'],

			'S_ADD_MOOD'	=> true,
			'S_ERROR'		=> (sizeof($errors)) ? true : false,
			)
		);
	}

	/**
	 * Delete a mood
	 *
	 * @param int $mood_id The mood identifier to delete
	 * @return null
	 * @access public
	 */
	public function delete_mood($mood_id)
	{
		if (confirm_box(true))
		{
			// Grab the mood name for the log...
			$sql = 'SELECT mood_name, mood_image
				FROM ' . $this->moods_table . '
				WHERE mood_id = ' .(int) $mood_id;
			$result = $this->db->sql_query($sql);
			$mood_row = $this->db->sql_fetchrow($result);
			$this->db->sql_freeresult($result);

			// Delete the mood...
			$sql = 'DELETE FROM ' . $this->moods_table . '
				WHERE mood_id = ' . (int) $mood_id;
			$this->db->sql_query($sql);

			// Reset the mood for users
			$sql = 'UPDATE ' . USERS_TABLE . '
				SET user_mood = 0
				WHERE user_mood = ' . (int) $mood_id;
			$this->db->sql_query($sql);

			$this->log->add('admin', $this->user->data['user_id'], $this->user->ip, 'LOG_MOODS_DELETED', time(), array($mood_row['mood_name']));

			$this->cache->destroy('_user_moods');
			$this->functions->cache_moods();

			trigger_error($this->user->lang['MSG_MOODS_DELETED'] . adm_back_link($this->u_action . "&amp;mode=manage"));
		}
		else
		{
			// display a count of users who have this mood
			$sql = 'SELECT COUNT(user_mood) AS mood_count
				FROM ' . USERS_TABLE . '
				WHERE user_mood = ' . (int) $mood_id;
			$result = $this->db->sql_query($sql);
			$row = $this->db->sql_fetchrow($result);
			$this->db->sql_freeresult($result);

			$message = $this->user->lang['MSG_CONFIRM'];
			if (!empty($row['mood_count']))
			{
				$message .= $row['mood_count'] <> 1 ? $this->user->lang('MSG_MOODS_CONFIRM_DELETE', $row['mood_count']) : $this->user->lang('MSG_MOOD_CONFIRM_DELETE', $row['mood_count']);
			}
			confirm_box(false, $message, build_hidden_fields(array(
				'id'		=> (int) $mood_id,
				'mode'		=> 'manage',
				'action'	=> 'delete'))
			);
			// Use a redirect to take the user back to the previous page
			// if the user chose not delete the mood from the confirmation page.
			redirect("{$this->u_action}");
		}
	}

	/* check mood
	*
	* a function to run mood validation on
	* @param string	$form_key	The forum key add_mood/edit_mood
	* @param string	$mood_image	The mood image
	* @param string	$mood_name	The mood name
	* @param array	$errors		The possible generated errors
	* @return array
	* @access private
	*/
	private function check_mood($mood_image, $mood_name, $errors, $form_key = '')
	{
		if (!check_form_key($form_key))
		{
			$errors[] = $this->user->lang['FORM_INVALID'];
		}

		if (empty($mood_name))
		{
			$errors[] = $this->user->lang['MOOD_ERROR_NO_MOOD_NAME'];
		}

		if (empty($mood_image))
		{
			$errors[] = $this->user->lang['MOOD_ERROR_NO_MOOD_IMG'];
		}

		if ($form_key == 'add_mood')
		{
			//we don't want two moods with the same name...right?
			$sql = 'SELECT mood_name
				FROM ' . $this->moods_table . "
				WHERE upper(mood_name) = '" . $this->db->sql_escape(strtoupper($mood_name)) . "'";
			$result = $this->db->sql_query($sql);

			if ($this->db->sql_fetchrow($result))
			{
				$errors[] = $this->user->lang['MOOD_NAME_EXISTS'];
			}
			$this->db->sql_freeresult($result);
		}

		return $errors;
	}

	/**
	 * Set page url
	 *
	 * @param string $u_action Custom form action
	 * @return null
	 * @access public
	 */
	public function set_page_url($u_action)
	{
		$this->u_action = $u_action;
	}
}
